/***
 * OW2 FraSCAti Tinfi
 * Copyright (C) 2010-2012 Inria, Univ. Lille 1
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: frascati@ow2.org
 *
 * Author: Lionel Seinturier
 */

package org.ow2.frascati.tinfi.osgi;

import java.io.IOException;
import java.util.List;

import org.objectweb.fractal.api.Type;
import org.objectweb.fractal.api.type.InterfaceType;
import org.objectweb.fractal.juliac.Juliac;
import org.objectweb.fractal.juliac.desc.ControllerDesc;
import org.objectweb.fractal.juliac.desc.MembraneDesc;
import org.objectweb.fractal.juliac.opt.InitializerClassGenerator;
import org.objectweb.fractal.juliac.osgi.FCOOCtrlGenerator;
import org.objectweb.fractal.juliac.proxy.ProxyClassGeneratorItf;

/**
 * Source code generator for <code>scaOsgiPrimitive</code> components.
 * 
 * @author Lionel Seinturier <Lionel.Seinturier@univ-lille1.fr>
 * @since 1.3
 */
public class FCOOCtrlSourceCodeGenerator
extends org.ow2.frascati.tinfi.opt.oo.FCOOCtrlSourceCodeGenerator {
    
    // -----------------------------------------------------------------------
    // Implementation of the JuliacModuleItf interface
    // -----------------------------------------------------------------------

	private FCOOCtrlGenerator osgig;

    /** The Fractal controller descriptor for Tinfi OSGi primitive components. */
    public static final String SCA_OSGI_PRIMITIVE = "scaOsgiPrimitive";

    public void init( Juliac jc ) throws IOException {
    	super.init(jc);
        osgig = new FCOOCtrlGenerator(jc);
    }
    
    @Override
	protected void postInit() throws IOException {
        super.postInit();
        mloader.put(SCAOSGiPrimitive.NAME,SCAOSGiPrimitive.class);
	}

    
    // -----------------------------------------------------------------------
    // Implementation of the FCSourceCodeGeneratorItf interface
    // -----------------------------------------------------------------------

    /**
     * Return <code>true</code> if the current source code generator handles the
     * specified controller descriptor.
     * 
     * @param controllerDesc  the controller descriptor to be checked
     * @return
     * 		<code>true</code> if <code>controllerDesc</code> should be accepted 
     */
    @Override
	public boolean acceptCtrlDesc( Object controllerDesc ) {
        boolean accept = controllerDesc.equals(SCA_OSGI_PRIMITIVE);
        return accept;
	}

    /**
     * Generate the source code associated to a Fractal component.
     * The membrane implementation and the initializer implementation are
     * generated.
     * 
     * @param type            the component type
     * @param controllerDesc  the component controller descriptor
     * @param contentDesc     the component content descriptor
     * @return  the initializer class generator for this component
     */
    @Override
	public InitializerClassGenerator<List<ControllerDesc>> generate(
		Type type, Object controllerDesc, Object contentDesc )
	throws IOException {
		
        /*
         * Check parameter consistence.
         */
    	osgig.check(type,controllerDesc,contentDesc);
        
		InitializerClassGenerator<List<ControllerDesc>> scg =
			super.generate(type,controllerDesc,contentDesc);
		
		return scg;
	}

    /**
     * Return the source code generator for component interfaces.
     */
    @Override
    public ProxyClassGeneratorItf getInterfaceClassGenerator( 
        InterfaceType it, MembraneDesc<?> membraneDesc ) {

    	ProxyClassGeneratorItf pcg =
    		osgig.getInterfaceClassGenerator(it,membraneDesc);
    	return pcg;
    }
    
    /**
     * Return the source code generator for the initializer class associated
     * with this component source code generator.
     */
	@Override
    public InitializerOOCtrlClassGenerator getInitializerClassGenerator() {
    	return new InitializerOOCtrlClassGenerator();
    }

    // ------------------------------------------------------------------
    // Implementation of methods defined in FCSourceCodeGenerator
    // ------------------------------------------------------------------
    
    @Override
    protected void generateInterfaceImpl( InterfaceType[] its, String ctrlDesc )
    throws IOException {

        for (InterfaceType it : its) {
            
            generateInterfaceImpl(it,ctrlDesc);
            
            /*
             * Super method generate internal interfaces. We don't need them for
             * scaOsgiPrimitive.
             */
        }
    }
    
    /**
     * Generate the source code of component interfaces associated with the
     * specified interface type.
     */
    @Override
    protected void generateInterfaceImpl( InterfaceType it, String ctrlDesc )
    throws IOException {
    	super.generateInterfaceImpl(it,ctrlDesc);
        osgig.generateInterfaceImpl(it,ctrlDesc);
    }    
}
